/*============================================================================
  File:     01_FindingQueries.sql

  SQL Server Versions: 2016, 2017
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2018, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Restore database
	*you may need to change the backup
	and restore locations
*/
USE [master]
GO

RESTORE DATABASE [WideWorldImporters] 
	FROM  DISK = N'C:\Backups\WideWorldImporters_Bits.bak' WITH  FILE = 1,  
	MOVE N'WWI_Primary' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.mdf',  
	MOVE N'WWI_UserData' TO N'C:\Databases\WideWorldImporters\WideWorldImporters_UserData.ndf',  
	MOVE N'WWI_Log' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.ldf',  
	NOUNLOAD,  REPLACE,  STATS = 5

GO


/*
	Enable query store, configure settings

*/
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE = ON;
GO
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE, QUERY_CAPTURE_MODE = ALL, INTERVAL_LENGTH_MINUTES = 1
	);
GO

/*
	Optional: Clear out anything that may be in there
*/
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE CLEAR;
GO

/*
	Run a query
*/
USE [WideWorldImporters];
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [ValidFrom] >= '2018-01-01';
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [PersonID] = 2;
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [PersonID] = 54685;
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [PreferredName] LIKE 'B%';
GO

EXECUTE sp_executesql   
          N'SELECT * FROM [Application].[People]   
          WHERE [PreferredName] LIKE @name',  
          N'@name nvarchar(20)',
		  @name = 'Ha%'

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [PreferredName] LIKE 'Rob%'
OPTION (QUERYTRACEON 4199);
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [PreferredName] LIKE 'A%'
OPTION (USE HINT ('ENABLE_PARALLEL_PLAN_PREFERENCE'));
GO

EXEC sp_query_store_flush_db;
GO


/*
	What do we get from the system views?
*/
USE [WideWorldImporters];
GO

SELECT *
FROM [sys].[query_store_query_text]
ORDER BY [query_text_id] ASC;
GO

SELECT *
FROM [sys].[query_store_query]
ORDER BY [query_text_id] ASC;
GO

SELECT *
FROM [sys].[query_store_plan];
GO

SELECT *
FROM [sys].[query_store_runtime_stats];
GO


/*
	What do we see in Query Store?
*/
SELECT
	[qsq].[query_id], 
	[qsp].[plan_id],
	[qsq].[object_id],
	[rs].[count_executions],
	[qst].[query_sql_text],
	TRY_CONVERT(XML, [qsp].[query_plan]),
	[qsp].[query_plan]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
--WHERE [qst].[query_sql_text] NOT LIKE 'SELECT StatMan%'
ORDER BY [qst].[query_text_id] DESC;
GO



/*
	Textual matching still in effect
*/
SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [SearchName] LIKE 'E%';
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [searchname] LIKE 'E%';
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE        [SearchName] LIKE 'E%';
GO

SELECT [FullName], [PreferredName]
FROM [Application].[People]
WHERE [SearchName] LIKE 'E%'
OPTION (RECOMPILE);
GO


SELECT
	[qst].[query_text_id],
	[qsq].[query_id], 
	[qsp].[plan_id], 
	[qsq].[object_id],  
	DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), 
		[qsp].[last_execution_time]) AS [LocalLastExecutionTime],
	[qsq].[query_hash],
	[qst].[query_sql_text], 
	ConvertedPlan = TRY_CONVERT(XML, [qsp].[query_plan])
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
WHERE [qst].[query_sql_text] LIKE '%SearchName%';
GO

/*
	Note some of these queries have the query_hash
*/
SELECT [query_hash] AS [query_hash], COUNT([query_id]) AS [Count]
FROM [sys].[query_store_query] 
GROUP BY [query_hash]
HAVING COUNT(*) > 1;
GO

SELECT
	[qst].[query_text_id],
	[qsq].[query_id], 
	[qsp].[plan_id], 
	[qsq].[object_id],  
	DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), 
		[qsp].[last_execution_time]) AS [LocalLastExecutionTime],
	[qsq].[query_hash],
	[qst].[query_sql_text], 
	ConvertedPlan = TRY_CONVERT(XML, [qsp].[query_plan])
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN (
	SELECT [query_hash] AS [query_hash], COUNT([query_id]) AS [Count]
	FROM [sys].[query_store_query] 
	GROUP BY [query_hash]
	HAVING COUNT(*) > 1
	) [m] on [qsq].[query_hash] = [m].[query_hash];
GO

/*
	Individual QUERIES are captured
*/
USE [WideWorldImporters];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_GetCustomerDetail];
GO

CREATE PROCEDURE [Sales].[usp_GetCustomerDetail]
	@CustomerName NVARCHAR(100)
AS	
	
	CREATE TABLE #CustomerList (
		[RowID] INT IDENTITY (1,1),
		[CustomerID] INT,
		[CustomerName] NVARCHAR (100)
		);

	INSERT INTO #CustomerList (
		[CustomerID], 
		[CustomerName]
		)
	SELECT 
		[CustomerID], 
		[CustomerName]
	FROM [Sales].[Customers]
	WHERE [CustomerName] LIKE @CustomerName
	UNION
	SELECT 
		[CustomerID], 
		[CustomerName]
	FROM [Sales].[Customers_Archive]
	WHERE [CustomerName] LIKE @CustomerName;

	SELECT 
		[o].[CustomerID], 
		[o].[OrderID],
		[il].[InvoiceLineID],
		[o].[OrderDate], 
		[i].[InvoiceDate],
		[ol].[StockItemID], 
		[ol].[Quantity],
		[ol].[UnitPrice],
		[il].[LineProfit]
	INTO #CustomerOrders
	FROM [Sales].[Orders] [o]
	INNER JOIN [Sales].[OrderLines] [ol] 
		ON [o].[OrderID] = [ol].[OrderID]
	INNER JOIN [Sales].[Invoices] [i]	
		ON [o].[OrderID] = [i].[OrderID]
	INNER JOIN [Sales].[InvoiceLines] [il] 
		ON [i].[InvoiceID] =  [il].[InvoiceID]		
		AND [il].[StockItemID] = [ol].[StockItemID]
		AND [il].[Quantity] = [ol].[Quantity]
		AND [il].[UnitPrice] = [ol].[UnitPrice]
	WHERE [o].[CustomerID] IN (SELECT [CustomerID] FROM #CustomerList);

	SELECT 
		[cl].[CustomerName],
		[si].[StockItemName],
		SUM([co].[Quantity]) AS [QtyPurchased],
		SUM([co].[Quantity]*[co].[UnitPrice]) AS [TotalCost],
		[co].[LineProfit],
		[co].[OrderDate],
		DATEDIFF(DAY,[co].[OrderDate],[co].[InvoiceDate]) AS [DaystoInvoice]
	FROM #CustomerOrders [co]
	INNER JOIN #CustomerList [cl]
		ON [co].[CustomerID] = [cl].[CustomerID]
	INNER JOIN [Warehouse].[StockItems] [si]
		ON [co].[StockItemID] = [si].[StockItemID]
	GROUP BY [cl].[CustomerName], [si].[StockItemName],[co].[InvoiceLineID], [co].[LineProfit], [co].[OrderDate], DATEDIFF(DAY,[co].[OrderDate],[co].[InvoiceDate])
	ORDER BY [co].[OrderDate];

GO


EXEC [Sales].[usp_GetCustomerDetail] N'Alvin Bollinger';
GO 10

EXEC [Sales].[usp_GetCustomerDetail] N'Tami Braggs';
GO 10

EXEC [Sales].[usp_GetCustomerDetail] N'Logan Dixon';
GO 10

EXEC [Sales].[usp_GetCustomerDetail] N'Tara Kotadia';
GO 10

/*
	What do we see in QS
*/
SELECT
	[qsq].[query_id], 
	[qsp].[plan_id], 
	[qsq].[object_id], 
	[qst].[query_sql_text], 
	ConvertedPlan = TRY_CONVERT(XML, [qsp].[query_plan])
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
WHERE [qsq].[object_id] = OBJECT_ID(N'Sales.usp_GetCustomerDetail');
GO

/*
	Include runtime stats
*/
SELECT
	[qsq].[query_id], 
	[qsp].[plan_id], 
	[qsq].[object_id], 
	[rs].[runtime_stats_interval_id],
	[rsi].[start_time],
	[rsi].[end_time],
	[rs].[count_executions],
	[rs].[avg_duration],
	[rs].[avg_cpu_time],
	[rs].[avg_logical_io_reads],
	[rs].[avg_rowcount],
	[qst].[query_sql_text], 
	ConvertedPlan = TRY_CONVERT(XML, [qsp].[query_plan])
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
JOIN [sys].[query_store_runtime_stats_interval] [rsi]
	ON [rs].[runtime_stats_interval_id] = [rsi].[runtime_stats_interval_id]
WHERE [qsq].[object_id] = OBJECT_ID(N'Sales.usp_GetCustomerDetail')
AND [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
AND [rs].[execution_type] = 0
ORDER BY [qsq].[query_id], [qsp].[plan_id], [rs].[runtime_stats_interval_id];
GO


/*
	Run external script to increase execution count
*/


/*
	Look at execution statistics again
*/
SELECT
	[qsq].[query_id], 
	[qsp].[plan_id], 
	[qsq].[object_id], 
	[rs].[runtime_stats_interval_id],
	[rsi].[start_time],
	[rsi].[end_time],
	[rs].[count_executions],
	[rs].[avg_duration],
	[rs].[avg_cpu_time],
	[rs].[avg_logical_io_reads],
	[rs].[avg_rowcount],
	[qst].[query_sql_text], 
	ConvertedPlan = TRY_CONVERT(XML, [qsp].[query_plan])
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
JOIN [sys].[query_store_runtime_stats_interval] [rsi]
	ON [rs].[runtime_stats_interval_id] = [rsi].[runtime_stats_interval_id]
WHERE [qsq].[object_id] = OBJECT_ID(N'Sales.usp_GetCustomerDetail')
AND [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
AND [rs].[execution_type] = 0
ORDER BY [qsq].[query_id], [qsp].[plan_id], [rs].[runtime_stats_interval_id];
GO


/*
	Compare query perf for all queries in the SP
*/
SELECT
	[qsq].[query_id], 
	[qsp].[plan_id], 
	OBJECT_NAME([qsq].[object_id]) AS [ObjectName], 
	SUM([rs].[count_executions]) AS [TotalExecutions],
	AVG([rs].[avg_duration]) AS [Avg_Duration],
	AVG([rs].[avg_cpu_time]) AS [Avg_CPU],
	AVG([rs].[avg_logical_io_reads]) AS [Avg_LogicalReads],
	MIN([qst].[query_sql_text]) AS[Query]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
JOIN [sys].[query_store_runtime_stats_interval] [rsi]
	ON [rs].[runtime_stats_interval_id] = [rsi].[runtime_stats_interval_id]
WHERE [qsq].[object_id] = OBJECT_ID(N'Sales.usp_GetCustomerDetail')
AND [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
AND [rs].[execution_type] = 0
GROUP BY [qsq].[query_id], [qsp].[plan_id], OBJECT_NAME([qsq].[object_id])
ORDER BY AVG([rs].[avg_cpu_time]) DESC;
GO

/*
	What's in the UI?
	Kick off other scripts
*/


/*
	Query to see which plan in cache
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
SELECT 
	[qs].[creation_time],
	[qs].execution_count, 
	[s].[text], 
	[qs].[query_hash], 
	[qs].[query_plan_hash], 
	[qp].[query_plan]
FROM sys.dm_exec_query_stats AS [qs]
CROSS APPLY sys.dm_exec_query_plan ([qs].[plan_handle]) AS [qp]
CROSS APPLY sys.dm_exec_sql_text([qs].[plan_handle]) AS [s]
INNER JOIN sys.dm_exec_cached_plans AS [cp] 
	ON [qs].[plan_handle] = [cp].[plan_handle]
WHERE [s].[text] LIKE '%CustomerTransactionInfo%';
GO
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
GO
